/* dwitest.c
   Read sync info and step data in a DWI step chart, testing the
   code in kadchart.c, dwiparse.c, and sync_compile.c

Copyright 2004 Damian Yerrick

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, provided that the above copyright notice(s) and this permission notice appear in all copies of the Software and that both the above copyright notice(s) and this permission notice appear in supporting documentation.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

Except as contained in this notice, the name of a copyright holder shall not be used in advertising or otherwise to promote the sale, use or other dealings in this Software without prior written authorization of the copyright holder.

*/



#include "dwiparse.h"

static const char dwi_column_names[8][4] =
{
  "1L","1D","1U","1R","2L","2D","2U","2R"
};


static const char difficulty_types[][16] =
{
  "edit", "beginner", "light", "standard", "heavy", "oni"
};

static void dwi_print_chart(const KADChart *c)
{
  size_t i;
  unsigned int active_freeze = 0;
  unsigned int difficulty_type = c->difficulty_type;
  const KADParsedRow *steps = c->steps->data;
  const size_t n = c->steps->num;

  if(difficulty_type > 5)
    difficulty_type = 0;

  fputs("== ", stdout);
  switch(c->steps_type)
  {
  case 'S':
    fputs("single", stdout);
    break;
  case 'C':
    fputs("couple", stdout);
    break;
  case 'D':
    fputs("double", stdout);
    break;
  case '6':
    fputs("solo", stdout);
    break;
  default:
    fputs("unknown", stdout);
    break;
  }
  printf(" %s steps (%u'), %u rows ==\n",
         difficulty_types[difficulty_type],
         (unsigned int)c->difficulty,
         n);

  for(i = 0; i < n; i++)
  {
    unsigned char c = steps[i].column;
    unsigned int row = steps[i].row;
    unsigned int beat = row / 48;


    printf("%3u:%u:%2u:", steps[i].measure + 1, beat + 1, row - 48 * beat);
    if((c & 0x7f) <= 8)
      fputs(dwi_column_names[(c & 0x7f) - 1], stdout);
    else
      printf("%2d", c & 0x7f);
    if(c & 0x80)
    {
      fputs(" FREEZE!", stdout);
      active_freeze |= 1 << (c & 0x7f);
    }
    else if(active_freeze & (1 << c))
    {
      active_freeze &= ~(1 << c);
      fputs(" LET GO", stdout);
    }
    fputc('\n', stdout);
  }
}


static void print_sync(const KADSync *sync)
{
  size_t i = 0;
  const KADBPMChange *bpms;
  const KADStop *stops;

  if(!sync)
    return;

  printf("BPM range: %.2f..%.2f BPM",
         KADUNFIX(sync->min_real_speed),
         KADUNFIX(sync->max_real_speed));
  if(sync->max_disp_speed == 0)
  {
    fputs(" (displayed as haywire)", stdout);
  }
  else if(sync->min_disp_speed <= sync->max_disp_speed)
  {
    printf(" (displayed as %.2f..%.2f BPM)",
           KADUNFIX(sync->min_disp_speed),
           KADUNFIX(sync->max_disp_speed));
  }
  printf("\nSample starts at %.2f s and plays for %.2f s\n",
         KADUNFIX(sync->sample_start),
         KADUNFIX(sync->sample_duration));

  bpms = sync->bpms->data;
  for(i = 0; i < sync->bpms->num; i++, bpms++)
  {
    unsigned int measure = bpms->row / 192;
    unsigned int row = bpms->row - 192 * measure;
    unsigned int beat = row / 48;

    printf("Tempo change: %3u:%u:%02u = %.4f BPM\n",
           measure + 1, beat + 1, row - 48 * beat,
           KADUNREAL(bpms->bpm));
  }

  stops = sync->stops->data;
  for(i = 0; i < sync->stops->num; i++, stops++)
  {
    unsigned int measure = stops->row / 192;
    unsigned int row = stops->row - 192 * measure;
    unsigned int beat = row / 48;

    printf("Stop: %3u:%u:%02u = %.1f ms\n",
           measure + 1, beat + 1, row - 48 * beat,
           1000. * KADUNREAL(stops->duration));
  }
}


static void print_song(const KADParsedSong *song)
{
  const KADChart **charts = song->charts->data;
  unsigned int i;

  printf("Title:  %s\nArtist: %s\n",
         song->title, song->artist);
  print_sync(song->sync);
  for(i = 0; i < song->charts->num; i++)
  {
    dwi_print_chart(charts[i]);
  }
}

KADV *KADSync_compile(KADSync *sync);

/* run as dwiparse < lom.dwi */
int main(void)
{
  KADParsedSong *song = KADParsedSong_new();
  KADV *compile_test;
  if(!song)
    return EXIT_FAILURE;
  KADParsedSong_ReadDWI(song, stdin);
  puts("=== Parsed content of DWI file ===");
  print_song(song);

  puts("=== Speed changes ===");
  compile_test = KADSync_compile(song->sync);
  if(compile_test)
    KADV_delete(compile_test);

  KADParsedSong_delete(song);
  return 0;
}

